#define _CRT_SECURE_NO_WARNINGS
#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>
#include <map>
#include <fstream>
#include <iostream>
#include <cmath>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <string>
#include <sstream>
#include <bitset>
#include <stack>
#include <deque>

#define all(x) x.begin(),x.end()
#define mpair make_pair

typedef long long ll;
typedef long double ld;

const ld epsylon = 1e-9;

using namespace std;

struct ship {
	int speed;
	int start_planet;
	int cap;
	int price;
};
struct flight {
	int from, to;
	int dist;
	int start_time;
	int end_time;
	int earn;
	int num_passengers;
	int month;
};
const int invalid_score = -100000000;
template <class T>
string to_s(const vector<T>& v, string delim = " ") {
	ostringstream out;
	for (unsigned i=0;i<v.size(); ++i){
		out << v[i];
		if (i + 1 != v.size()) {
			out << delim;
		}
	}
	return out.str();
}

struct schedule_item {
	int time_to_travel; // in hours
	int day;
	int flight_index;

	int dist;
	int start_time;
	int from, to;
	int GetFrom(const vector<flight>& b) const {
		if (flight_index == -1) {
			return from;
		} else {
			return b[flight_index].from;
		}
	}
	int GetTo(const vector<flight>& b) const {
		if (flight_index == -1) {
			return to;
		} else {
			return b[flight_index].to;
		}
	}
	int GetStartTime(const vector<flight>& b) const {
		if (flight_index == -1) {
			return start_time;
		} else {
			return b[flight_index].start_time;
		}
	}
	int GetDist(const vector<flight>& b) const {
		if (flight_index == -1) {
			return dist;
		} else {
			return b[flight_index].dist;
		}
	}
};

int N;
int distances[64][64];
int planet_hour_other_planet[64][20][64];
ll gain[64][17][64][17];
int upto[64][20][64];

int get_time(int d, int v) {
	return (d + v - 1)/v;
}
ll get_score(int number_of_days, const schedule_item& si, const vector<flight>& b, const ship& sh) {
	if (si.flight_index == -1) {
		ll d = si.GetDist(b);
		return -1 * ((64LL *4LL) /number_of_days) * d * sh.price;
	} else {
		const flight& fl = b[si.flight_index];
		ll start = fl.month*64LL + 1LL;
		ll end = start + 64LL;
		ll res = -1LL * ((64LL *4LL) /number_of_days) * fl.dist * sh.price;
		res += ((end - start)/number_of_days) *  min(sh.cap, fl.num_passengers) * fl.earn;
		return res;
	}
}
ll get_score(const vector<ship>& a, const vector<flight>& b, const vector<vector<schedule_item> >& res) {
	if (res.size() != a.size()) {
		cerr << "size of res differs from size of a!\n";
		return invalid_score;
	}
	int used[64][64][20];
	memset(used, 0, sizeof(used));
	vector<int> number_of_days(res.size());
	for (unsigned i=0;i<res.size();++i) {
		number_of_days[i] = res[i].back().day + 1;	
	}
	ll score = 0;

	for (unsigned i = 0; i < res.size(); ++i) {
		int current_time = 4;
		int current_day = 1;
		if (res[i].back().GetTo(b) != a[i].start_planet) {
			cerr << "You did not return to your home planet\n";
			return invalid_score;
		}
		if (res[i][0].GetFrom(b) != a[i].start_planet) {
			cerr << "Does not start from the start planet!\n";
			return invalid_score;
		}
		for (unsigned j = 1; j < res[i].size();++j) {
			if (distances[res[i][j].GetFrom(b)][res[i][j].GetTo(b)] == -1) {
				cerr << "a flight between forbidden planets is attempted\n";
				return invalid_score;
			}
		} 
		for (unsigned j  = 0; j < res[i].size(); ++j) {
			if (j == 0 || res[i][j].day != res[i][j-1].day) {
				current_time = 4;
			}
			if (j > 0 && res[i][j].day < res[i][j-1].day) {
				cerr << "The days are not ordered!\n";
				return invalid_score;
			}
			if (j > 0 && res[i][j].GetFrom(b) != res[i][j-1].GetTo(b)) {
				cerr << "The cycle is not valid\n";
				return invalid_score;
			}
			
			if (res[i][j].GetStartTime(b) < current_time) {
				cerr << "Sorry but you can not use flight [" << res[i][j].flight_index << "] on day [" << res[i][j].day << "] for flight [" << res[i][j].flight_index << "]\n";
				return invalid_score;
			}
			if (used[res[i][j].GetFrom(b)][res[i][j].GetTo(b)][res[i][j].GetStartTime(b)]) {
				cerr << "More then once travelling from " << res[i][j].GetFrom(b)<< " to " << res[i][j].GetTo(b) << " at " << res[i][j].GetStartTime(b) << endl;
				cerr << i << endl;
				cerr << j << endl;

				return invalid_score;
			}
			used[res[i][j].GetFrom(b)][res[i][j].GetTo(b)][res[i][j].GetStartTime(b)] = 1;
			score += get_score(number_of_days[i], res[i][j], b, a[i]);
			int st =  res[i][j].GetStartTime(b);
			int tt = get_time(res[i][j].GetDist(b), a[i].speed);
			if (st + tt > 16) {
				cerr << "A plane [" <<	i << "]  did not land at the end of the day\n";
			}
			current_time = st + tt;
		}
	}

	return score;
}
vector<int> pos[64][20][64];

ll return_as_cheap_as_possible(const vector<ship>& a, const vector<flight>& b, vector<vector<schedule_item> >& res, int index) {
	vector<bool> used(b.size(), false);
	memset(planet_hour_other_planet, 0, sizeof(planet_hour_other_planet));
	memset(gain, 0, sizeof(planet_hour_other_planet));
	for (unsigned i = 0; i < a.size(); ++i) {
		for (unsigned j = 0; j< res[i].size();++j) {
			planet_hour_other_planet[res[i][j].GetFrom(b)][res[i][j].GetStartTime(b)][res[i][j].GetTo(b)] = 1;
		}
	}
	for (int i =0;i<64;++i){
		for (int j=0;j<20;++j){
			for (int k =0;k<64;++k){
				pos[i][j][k].clear();
			}
		}
	}
	for (unsigned i =0 ; i < b.size(); ++i) {
		int req = get_time(b[i].dist, a[index].speed);
		pos[b[i].from][b[i].start_time][b[i].to].push_back(i);
		for (int l = b[i].start_time +req; l <= b[i].end_time; ++l){
			gain[b[i].from][b[i].start_time][b[i].to][l] += min((ll)a[index].cap, (ll)b[i].num_passengers) * (ll)b[i].earn;
		}
	}
	int from; int to ;
	int time1, time2;
	ll sc = -1;
	bool first = true;
	for (int i = a[index].start_planet, jt= 0; jt<1 ; ++jt) {
		for (int j=0;j<N;++j) {
			if (j == i) continue;
			ll d1 = distances[i][j];
			ll d2 = distances[j][i];
			if (d1 == -1) {
				continue;
			}
			int tm = get_time((int)d1, a[index].speed);
			int tm1 = -1;
			int tm2 = -1;
			ll e1 = 0, e2 = 0;
			for (int  k = 4; k + tm <= 16; ++k) {
				if (planet_hour_other_planet[i][k][j] == 0) {
					if (tm1 == -1 || gain[i][k][j][k+tm] > e1) {
						tm1 =  k;
						e1 = gain[i][k][j][k+tm];
					}
				}
			}
			if (tm1 == -1) {
				continue;
			}
			int ttm = get_time((int)d2, a[index].speed);
			for (int  k = 4; k + ttm <= 16; ++k) {
				if (planet_hour_other_planet[j][k][i] == 0) {
					if (tm2 == -1 || gain[j][k][i][k+ttm] > e2) {
						tm2 =  k;
						e2 = gain[j][k][i][k+ttm];
					}
				}
			}
			if (tm2 == -1) {
				continue;
			}

			ll tempsc = 0;
			for (int ii = 1; ii <= 64 * 4; ++ii) {
				if (ii % 3 == 1) {
					tempsc -= (ll)d1 * (ll)a[index].price;
				} else if (ii % 3 == 2) {
					tempsc -= (ll)d2 * (ll)a[index].price;
				}
				if (ii % 3 == 1) {
					int req = get_time(distances[i][j], a[index].speed);
					if (tm1 + req <= 16) {
						for (unsigned l = 0; l < pos[i][tm1][j].size(); ++l) {
							int idx = pos[i][tm1][j][l];
							if (b[idx].end_time < tm1 + req) {
								continue;
							}
							if (ii >= (b[idx].month-1)*64 + 1 && ii <= (b[idx].month)*64) {
								tempsc += (ll)b[idx].earn * min((ll)a[index].cap, (ll)b[idx].num_passengers);
							}
						}
					}

				} else if (ii % 3 == 2) {
					int req = get_time(distances[j][i], a[index].speed);
					if (tm2 + req <= 16) {
						for (unsigned l = 0; l < pos[j][tm2][i].size(); ++l) {
							int idx = pos[j][tm2][i][l];
							if (b[idx].end_time < tm2 + req) {
								continue;
							}
							if (ii >= (b[idx].month-1)*64 + 1 && ii <= (b[idx].month)*64) {
								tempsc += (ll)b[idx].earn * min((ll)a[index].cap, (ll)b[idx].num_passengers);
							}
						}
					}
				}
			}
			// ll tempsc = ((64LL*4LL)/3LL) * ((ll)d1 + (ll)d2)*(-(ll)a[index].price) + ((ll)e1 + (ll)e2)*11LL;
			if (first || tempsc > sc) {
				sc = tempsc;
				from = i;
				to = j;
				time1 = tm1;
				time2 = tm2;
				first = false;
			}
		}
	}
	if (first == true) {
		return -1;
	}
	schedule_item temp;
	temp.from = from;
	temp.to = to;
	temp.dist = distances[from][to];
	temp.flight_index = -1;
	temp.start_time = time1;
	temp.day = 1;
	res[index].push_back(temp);
	temp.from = to;
	temp.to = from;
	temp.dist = distances[to][from];
	temp.flight_index = -1;
	temp.start_time = time2;
	temp.day = 2;
	res[index].push_back(temp);
	return sc;
}

//bool two_steps(const vector<ship>& a, const vector<flight>& b, vector<vector<schedule_item> >& res, int index) {
//	vector<bool> used(b.size(), false);
//	memset(planet_hour_other_planet, 0, sizeof(planet_hour_other_planet));
//	memset(gain, 0, sizeof(planet_hour_other_planet));
//	for (unsigned i = 0; i < a.size(); ++i) {
//		for (unsigned j = 0; j< res[i].size();++j) {
//			planet_hour_other_planet[res[i][j].GetFrom(b)][res[i][j].GetStartTime(b)][res[i][j].GetTo(b)] = 1;
//		}
//	}
//	for (unsigned i =0 ; i < b.size(); ++i) {
//		gain[b[i].from][b[i].start_time][b[i].to] = min(a[index].cap, b[i].num_passengers) * b[i].earn;
//		upto[b[i].from][b[i].start_time][b[i].to] = b[i].end_time;
//	}
//	int from; int to ;
//	int time[4];
//	ll sc = -1;
//	bool first = true;
//	int o = a[index].start_planet;
//	for (int i = 0; i<N ; ++i) {
//		for (int j=0;j<N;++j) {
//			if (j == i || j == a[index].start_planet || j == a[index].start_planet) {
//				continue;
//			}
//			ll ds[4];
//			ds[0] = distances[o][i];
//			ds[1] = distances[i][j];
//			ds[2] = distances[j][i];
//			ds[3] = distances[i][o];
//
//			if (ds[0] == -1 || ds[1] == -1) {
//				continue;
//			}
//			int tms[4];
//			int e[4];
//			memset(tms, -1, sizeof(tms));
//			memset(e, -1, sizeof(e));
//			vector<pair<int, int> > tu;
//			tu.push_back(mpair(o,i));
//			tu.push_back(mpair(i,j));
//			tu.push_back(mpair(j,i));
//			tu.push_back(mpair(i,o));
//			bool ok = true;
//			for (int y =0 ; y<4;++y) {
//				int tm = get_time((int)ds[y], a[index].speed);
//				for (int  k = 4; k + tm <= 16; ++k) {
//					if (planet_hour_other_planet[tu[y].first][k][tu[y].second] == 0) {
//						if (tms[y] == -1 || (upto[tu[y].first][k][tu[y].second] < k +tm && gain[tu[y].first][k][tu[y].second] > e[y])) {
//							tms[y] =  k;
//							e[y] = gain[tu[y].first][k][tu[y].second];
//						}
//					}
//				}
//				if (tms[y] == -1) {
//					ok = false;
//					break;
//				}
//			}
//			if (!ok) {
//				continue;
//			}
//			
//			ll ts = 0;
//			for (int y=0;y<4;++y){
//				ts += -(ll)a[index].price * ds[y] + e[y];
//			}
//			if (first || ts > sc) {
//				sc = ts;
//				from = i;
//				to = j;
//				for (int  y= 0;y<4;++y){
//					time[y]=tms[y];
//				}
//				first = false;
//			}
//		}
//	}
//	if (first == true) {
//		return false;
//	}
//	schedule_item temp;
//	vector<pair<int, int> > tu;
//	tu.push_back(mpair(o,from));
//	tu.push_back(mpair(from,to));
//	tu.push_back(mpair(to,from));
//	tu.push_back(mpair(from,o));
//	for (int y=0;y<4;++y){
//		temp.from = tu[y].first;
//		temp.to = tu[y].second;
//		temp.dist = distances[temp.from][temp.to];
//		temp.start_time = time[y];
//		temp.day = y + 1;
//		temp.flight_index = -1;
//		res[index].push_back(temp);
//	}
//	return true;
//}
void solve(const vector<ship>& a, const vector<flight>& b, vector<vector<schedule_item> >& res) {
	res.clear();
	res.resize(a.size());
	ll score = 0;
	vector<pair<int, int> > help(a.size());
	for (int i =0 ;i<a.size();++i){
		help[i].first = a[i].price;
		help[i].second = i;
	}
	sort(all(help));
	reverse(all(help));
	for (unsigned i  =0 ; i < a.size();++i) {
		score += return_as_cheap_as_possible(a, b, res, help[i].second);
	}
	cerr << "Score: " << score << endl;
}

void print_solution(const vector<ship>& a, const vector<flight>& b, const vector<vector<schedule_item> >& res) {
	for (unsigned i = 0; i < res.size(); ++i) {
		cout << res[i].size() << endl;
		for (unsigned j = 0 ; j < res[i].size(); ++j) {
			cout << res[i][j].day << " "<< res[i][j].GetStartTime(b) << " " << res[i][j].GetTo(b) << endl;
		}
	}
}
int main() {
	freopen("flights.in","r",stdin);
	freopen("flights.out","w",stdout);
	vector<ship> a;
	vector<flight> b;
	vector<vector<schedule_item> > res;

	int n,m,f;
	cin  >> n >> m >> f;
	N = n;
	a.resize(m);
	for (unsigned i = 0 ; i < a.size(); ++i) {
		cin >> a[i].start_planet >> a[i].speed >> a[i].price >> a[i].cap;
	}
	flight temp;
	int k;
	memset(distances, -1, sizeof(distances));
	
	for (int j= 0 ; j < f; ++j){
		cin >> temp.from >> temp.to >> temp.dist >> k;

		distances[temp.from][temp.to] = temp.dist;
		for (int i =0;i<k;++i){
			cin >> temp.start_time >> temp.end_time >> temp.month >> temp.earn >> temp.num_passengers; 
			if (temp.from >= 4) {
				b.push_back(temp);
			}
		}
	}
	solve(a,b, res);
	cerr << "Score: " << get_score(a,b, res) << endl;
	print_solution(a, b, res);
	// res.clear();

	return 0;
}


// 2450679232
// 11532058 

// 41871282
// 95332896
// 26014016
// 37228985  

// 90715372
// 85881924